#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "update.h"
#include "callbacks.h"

extern GtkWidget *commander_window;
extern GtkWidget *porthole_window;
extern GtkWidget *color_window;
extern GtkWidget *status_window;
extern GtkWidget *aperture_window;
extern GtkWidget *file_selector_window;
extern GtkWidget *configure_window;
extern GtkWidget *about_window;
extern dc1394_camerainfo camera;
extern char* feature_op_list[NUM_FEATURES];
extern char* feature_list[NUM_FEATURES];
extern char* feature_frame_list[NUM_FEATURES];
extern char* feature_scale_list[NUM_FEATURES];
extern char* feature_man_list[NUM_FEATURES];
extern char* feature_auto_list[NUM_FEATURES];
extern char* feature_power_list[NUM_FEATURES];


void UpdateRange(GtkWidget* current_window, int feature)
{
  int err;
  dc1394bool_t test_boolean;

  // grab&set power status:
  err=dc1394_is_feature_on(camera.handle,camera.id,feature,&test_boolean);
  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(current_window, feature_power_list[feature]),test_boolean);

  // grab&set auto/man status, plus sensitive range in man mode:
  err=dc1394_is_feature_auto(camera.handle,camera.id,feature,&test_boolean);
  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(current_window, feature_auto_list[feature]),test_boolean);
  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(current_window, feature_man_list[feature]),!test_boolean);
  if (feature==FEATURE_WHITE_BALANCE)
    {
      gtk_widget_set_sensitive(lookup_widget(current_window, "whitebal_BU_scale"),!test_boolean);
      gtk_widget_set_sensitive(lookup_widget(current_window, "whitebal_RV_scale"),!test_boolean);
    }
  else
    gtk_widget_set_sensitive(lookup_widget(current_window, feature_scale_list[feature]),!test_boolean);

  // grab&set range value if readable:
  UpdateRangeValue(current_window,feature);

}

void UpdateRangeValue(GtkWidget* current_window, int feature)
{
  int err, value, valueBU, valueRV;
  dc1394bool_t test_boolean;
  GtkAdjustment* adj;

  // grab&set range value if readable:
  err=dc1394_can_read_out(camera.handle,camera.id,feature,&test_boolean);
  if (test_boolean)
    {
      if (feature==FEATURE_WHITE_BALANCE)
	{
	  err=dc1394_get_white_balance(camera.handle,camera.id,&valueBU,&valueRV);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(current_window, "whitebal_BU_scale")));
	  gtk_adjustment_set_value(adj, valueBU);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(current_window, "whitebal_RV_scale")));
	  gtk_adjustment_set_value(adj, valueRV);
	  
	}
      else
	{
	  err=dc1394_get_feature_value(camera.handle,camera.id,feature,&value);
	  adj=gtk_range_get_adjustment(GTK_RANGE (lookup_widget(current_window, feature_scale_list[feature])));
	  gtk_adjustment_set_value(adj, value);
	}
    }

}

void
UpdateFpsMenu()
{
  int err, mode, format;
  quadlet_t value;
  GtkWidget* fps;
  GtkWidget* fps_menu;
  GtkWidget* glade_menuitem;

  // select current format and set available framerates
  err=dc1394_get_video_mode(camera.handle, camera.id, &mode);
  err=dc1394_get_video_format(camera.handle, camera.id, &format);
  err=dc1394_query_supported_framerates(camera.handle, camera.id, format, mode, &value);

  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"fps"))); // remove previous menu

  fps = gtk_option_menu_new ();
  gtk_widget_ref (fps);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "fps", fps,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fps);
  gtk_box_pack_start (GTK_BOX (lookup_widget(commander_window,"vbox25")), fps, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (fps), 1);

  fps_menu = gtk_menu_new ();

  if  ( value & (0x1<<31) )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("1.875 fps"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_fps1875_activate),
			  NULL);
    }
  if  ( value & (0x1<<30) )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("3.75 fps"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_fps375_activate),
			  NULL);
    }
  if  ( value & (0x1<<29) )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("7.5 fps"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_fps75_activate),
			  NULL);
    }
  if  ( value & (0x1<<28) )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("15 fps"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_fps15_activate),
			  NULL);
    }
  if  ( value & (0x1<<27) )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("30 fps"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_fps30_activate),
			  NULL);
    }
  if  ( value & (0x1<<26) )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("60 fps"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (fps_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_fps60_activate),
			  NULL);
    }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (fps), fps_menu);

  // read current fps and make it current menu:
  // err=dc1394_get_video_framerate(camera.handle, camera.id, &framerate);

  // here we set the sensitiveness, AFTER the 'gtk_option_menu_set_menu' command:
  err=dc1394_get_trigger_mode(camera.handle, camera.id, &mode);
  if (mode==0) // if the trigger is in mode = 0 (internal)
    gtk_widget_set_sensitive (lookup_widget(commander_window,"fps"), TRUE);
  else
    gtk_widget_set_sensitive (lookup_widget(commander_window,"fps"), FALSE);

}

void
UpdateTriggerModeMenu()
{
  int err, max_mode;
  quadlet_t value;
  GtkWidget* trigger_mode;
  GtkWidget* trigger_mode_menu;
  GtkWidget* glade_menuitem;

  // query supported modes.
  // PERSONAL interpretation of the DC v1.2 standrad:
  //       trigger modes are available from 0 to the value present
  //       in the feature_characterisitcs register
  err=dc1394_query_feature_characteristics(camera.handle,camera.id,FEATURE_TRIGGER,&value);

  max_mode=( (value & (0xF << 12))>>12 );

  // reverse byte order:
  max_mode=((max_mode & 0x8)>>3)
          +((max_mode & 0x4)>>1)
          +((max_mode & 0x2)<<1)
          +((max_mode & 0x1)<<3);

  gtk_widget_destroy(GTK_WIDGET (lookup_widget(commander_window,"trigger_mode"))); // remove previous menu

  trigger_mode = gtk_option_menu_new ();
  gtk_widget_ref (trigger_mode);
  gtk_object_set_data_full (GTK_OBJECT (commander_window), "trigger_mode", trigger_mode,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (trigger_mode);
  gtk_box_pack_start (GTK_BOX (lookup_widget(commander_window,"vbox25")), trigger_mode, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (trigger_mode), 1);

  gtk_widget_set_sensitive (trigger_mode, TRUE); // here we set the sensitiveness

  trigger_mode_menu = gtk_menu_new ();

  if  ( max_mode>=0 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Internal"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode0_activate),
			  NULL);
    }

  if  ( max_mode>=1 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 1"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode1_activate),
			  NULL);
    }

  if  ( max_mode>=2 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 2"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode2_activate),
			  NULL);
    }

  if  ( max_mode>=3 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 3"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode3_activate),
			  NULL);
    }

  if  ( max_mode>=4 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 4"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode4_activate),
			  NULL);
    }

  if  ( max_mode>=5 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 5"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode5_activate),
			  NULL);
    }

  if  ( max_mode>=6 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 6"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode6_activate),
			  NULL);
    }

  if  ( max_mode>=7 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 7"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode7_activate),
			  NULL);
    }

  if  ( max_mode>=8 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 8"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode8_activate),
			  NULL);
    }

  if  ( max_mode>=9 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 9"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode9_activate),
			  NULL);
    }

  if  ( max_mode>=10 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 10"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode10_activate),
			  NULL);
    }

  if  ( max_mode>=11 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 11"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode11_activate),
			  NULL);
    }

  if  ( max_mode>=12 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 12"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode12_activate),
			  NULL);
    }

  if  ( max_mode>=13 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 13"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode13_activate),
			  NULL);
    }

  if  ( max_mode>=14 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 14"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode14_activate),
			  NULL);
    }
  if  ( max_mode>=15 )
    {
      glade_menuitem = gtk_menu_item_new_with_label (_("Mode 15"));
      gtk_widget_show (glade_menuitem);
      gtk_menu_append (GTK_MENU (trigger_mode_menu), glade_menuitem);
      gtk_signal_connect (GTK_OBJECT (glade_menuitem), "activate",
			  GTK_SIGNAL_FUNC (on_trigger_mode15_activate),
			  NULL);
    }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (trigger_mode), trigger_mode_menu);
  // set current mode here:
}
